@@ -57,7 +57,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/queue/red.cc,v 1.80 2004/10/28 23:35:37 haldar Exp $ (LBL)";
+     "@(#) $Header: /cvsroot/nsnam/ns-2/queue/red.cc,v 1.88 2007/10/23 06:55:54 seashadow Exp $ (LBL)";
 #endif
 
 #include <math.h>
@@ -93,7 +93,7 @@ REDQueue::REDQueue() {
 /*
  * modified to enable instantiation with special Trace objects - ratul
  */
-REDQueue::REDQueue(const char * trace) : link_(NULL), de_drop_(NULL), EDTrace(NULL), tchan_(0), idle_(1)
+REDQueue::REDQueue(const char * trace) : link_(NULL), de_drop_(NULL), EDTrace(NULL), tchan_(0), idle_(1), idletime_(0.0)
 {
 	initParams();
 	
@@ -110,7 +110,7 @@ REDQueue::REDQueue(const char * trace) : link_(NULL), de_drop_(NULL), EDTrace(NU
 	bind("thresh_", &edp_.th_min_pkts);		    // minthresh
 	bind("thresh_queue_", &edp_.th_min);
 	bind("maxthresh_", &edp_.th_max_pkts);	    // maxthresh
-	bind("minthresh_queue_", &edp_.th_max);
+	bind("maxthresh_queue_", &edp_.th_max);
 	bind("mean_pktsize_", &edp_.mean_pktsize);  // avg pkt size
 	bind("idle_pktsize_", &edp_.idle_pktsize);  // avg pkt size for idles
 	bind("q_weight_", &edp_.q_w);		    // for EWMA
@@ -126,6 +126,7 @@ REDQueue::REDQueue(const char * trace) : link_(NULL), de_drop_(NULL), EDTrace(NU
 	bind_bool("wait_", &edp_.wait);
 	bind("linterm_", &edp_.max_p_inv);
 	bind("mark_p_", &edp_.mark_p);
+	bind_bool("use_mark_p_", &edp_.use_mark_p);
 	bind_bool("setbit_", &edp_.setbit);	    // mark instead of drop
 	bind_bool("gentle_", &edp_.gentle);         // increase the packet
 						    // drop prob. slowly
@@ -235,7 +236,6 @@ void REDQueue::initParams()
 	edp_.th_max = 0.0;
 	edp_.th_max_pkts = 0.0;
 	edp_.max_p_inv = 0.0;
-	edp_.mark_p = 0.0;
 	edp_.q_w = 0.0;
 	edp_.adaptive = 0;
 	edp_.cautious = 0;
@@ -277,7 +277,7 @@ void REDQueue::reset()
 	 * accordingly.
 	 */
         if (link_) {
-		edp_.ptc = link_->bandwidth() / (8. * edp_.mean_pktsize);
+		edp_.ptc = link_->bandwidth() / (8.0 * edp_.mean_pktsize);
 		initialize_params();
 	}
 	if (edp_.th_max_pkts == 0) 
@@ -316,7 +316,7 @@ void REDQueue::reset()
 	edv_.lastset = 0.0;
 	if (edp_.gentle) {
 		edv_.v_c = ( 1.0 - edv_.cur_max_p ) / edp_.th_max;
-		edv_.v_d = 2 * edv_.cur_max_p - 1.0;
+		edv_.v_d = 2.0 * edv_.cur_max_p - 1.0;
 	}
 
 	idle_ = 1;
@@ -442,12 +442,18 @@ REDQueue::calculate_p_new(double v_ave, double th_max, int gentle, double v_a,
 		// p ranges from max_p to 1 as the average queue
 		// size ranges from th_max to twice th_max 
 		p = v_c * v_ave + v_d;
-	} else {
-		// p ranges from 0 to max_p as the average queue
-		// size ranges from th_min to th_max 
-		p = v_a * v_ave + v_b;
-		p *= max_p;
-	}
+        } else if (!gentle && v_ave >= th_max) { 
+                // OLD: p continues to range linearly above max_p as
+                // the average queue size ranges above th_max.
+                // NEW: p is set to 1.0 
+                p = 1.0;
+        } else {
+                // p ranges from 0 to max_p as the average queue
+                // size ranges from th_min to th_max 
+                p = v_a * v_ave + v_b;
+                // p = (v_ave - th_min) / (th_max - th_min)
+                p *= max_p; 
+        }
 	if (p > 1.0)
 		p = 1.0;
 	return p;
@@ -480,7 +486,7 @@ REDQueue::modify_p(double p, int count, int count_bytes, int bytes,
 		if (count1 * p < 1.0)
 			p = 0.0;
 		else if (count1 * p < 2.0)
-			p /= (2 - count1 * p);
+			p /= (2.0 - count1 * p);
 		else
 			p = 1.0;
 	} else {
@@ -490,7 +496,9 @@ REDQueue::modify_p(double p, int count, int count_bytes, int bytes,
 			p = 1.0;
 	}
 	if (bytes && p < 1.0) {
-		p = p * size / mean_pktsize;
+		p = (p * size) / mean_pktsize;
+		//p = p * (size / mean_pktsize);
+
 	}
 	if (p > 1.0)
 		p = 1.0;
@@ -551,7 +559,8 @@ REDQueue::drop_early(Packet* pkt)
 		edv_.count = 0;
 		edv_.count_bytes = 0;
 		hdr_flags* hf = hdr_flags::access(pickPacketForECN(pkt));
-		if (edp_.setbit && hf->ect() && edv_.v_prob1 < edp_.mark_p) { 
+		if (edp_.setbit && hf->ect() && 
+                     (!edp_.use_mark_p || edv_.v_prob1 < edp_.mark_p)) { 
 			hf->ce() = 1; 	// mark Congestion Experienced bit
 			// Tell the queue monitor here - call emark(pkt)
 			return (0);	// no drop
@@ -625,6 +634,7 @@ void REDQueue::enque(Packet* pkt)
 	 * of time we've been idle for
 	 */
 
+	/*  print_edp(); */
 	int m = 0;
 	if (idle_) {
 		// A packet that arrives to an idle queue will never
@@ -680,8 +690,9 @@ void REDQueue::enque(Packet* pkt)
 	curq_ = qlen;	// helps to trace queue during arrival, if enabled
 
 	if (qavg >= edp_.th_min && qlen > 1) {
-		if ((!edp_.gentle && qavg >= edp_.th_max) ||
-			(edp_.gentle && qavg >= 2 * edp_.th_max)) {
+		if (!edp_.use_mark_p && 
+			((!edp_.gentle && qavg >= edp_.th_max) ||
+			(edp_.gentle && qavg >= 2 * edp_.th_max))) {
 			droptype = DTYPE_FORCED;
 		} else if (edv_.old == 0) {
 			/* 
@@ -814,7 +825,7 @@ int REDQueue::command(int argc, const char*const* argv)
 			// set ptc now
 			link_ = del;
 			edp_.ptc = link_->bandwidth() /
-				(8. * edp_.mean_pktsize);
+				(8.0 * edp_.mean_pktsize);
 			edp_.delay = link_->delay();
 			if (
 			  (edp_.q_w <= 0.0 || edp_.th_min_pkts == 0 ||
@@ -869,7 +880,8 @@ int REDQueue::command(int argc, const char*const* argv)
 void
 REDQueue::trace(TracedVar* v)
 {
-	char wrk[500], *p;
+	char wrk[500];
+	const char *p;
 
 	if (((p = strstr(v->name(), "ave")) == NULL) &&
 	    ((p = strstr(v->name(), "prob")) == NULL) &&
@@ -908,6 +920,7 @@ void REDQueue::print_edp()
 	printf("max_p: %f, qw: %f, ptc: %f\n",
 		(double) edv_.cur_max_p, edp_.q_w, edp_.ptc);
 	printf("qlim: %d, idletime: %f\n", qlim_, idletime_);
+	printf("mark_p: %f, use_mark_p: %d\n", edp_.mark_p, edp_.use_mark_p);
 	printf("=========\n");
 }
 
