@@ -22,7 +22,7 @@
 #    specific prior written permission.
 # 
 
-# @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcl/lib/ns-default.tcl,v 1.354 2005/10/07 05:58:30 tomh Exp $
+# @(#) $Header: /cvsroot/nsnam/ns-2/tcl/lib/ns-default.tcl,v 1.383 2009/01/02 21:50:24 tom_henderson Exp $
 
 # THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 # ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -72,6 +72,9 @@ CMUTrace set duration_scaling_factor_ 3.0e4
 
 Scheduler/RealTime set maxslop_ 0.010; # max allowed slop b4 error (sec)
 
+Scheduler/Calendar set adjust_new_width_interval_ 10;	# the interval (in unit of resize times) we recalculate bin width. 0 means disable dynamic adjustment
+Scheduler/Calendar set min_bin_width_ 1e-18;		# the lower bound for the bin_width
+
 #
 # Queues and associated
 #
@@ -112,8 +115,6 @@ CMUPriQueue set debug_ false
 Queue/dsRED set numQueues_ 4
 Queue/dsRED set ecn_ 0
 
-Queue/XCP set spread_bytes_ 0
-
 # XXX Temporary fix XXX
 # support only xcp flows; set to 1 when supporting both tcp and xcp flows; temporary fix for allocating link BW between xcp and tcp queues until dynamic queue weights come into effect. This fix should then go away
 Queue/XCP set tcp_xcp_on_ 0  ;
@@ -127,7 +128,9 @@ Queue/RED set thresh_ 0
 Queue/RED set maxthresh_ 0
 # Queue/RED/maxthresh_ was changed on 12/29/01, for automatic configuration.
 Queue/RED set thresh_queue_ [Queue set limit_]
-Queue/RED set minthresh_queue_ [Queue set limit_]
+# Note from Sally: I don't see that thresh_queue_ is used for anything.
+Queue/RED set maxthresh_queue_ [Queue set limit_]
+# Note from Sally: I don't see that maxthresh_queue_ is used for anything.
 Queue/RED set mean_pktsize_ 500
 Queue/RED set idle_pktsize_ 100
 # Queue/RED set q_weight_ 0.002
@@ -136,6 +139,9 @@ Queue/RED set q_weight_ -1
 Queue/RED set wait_ true
 Queue/RED set linterm_ 10
 Queue/RED set mark_p_ 0.1
+Queue/RED set use_mark_p_ true
+# Queue/RED/use_mark_p_ was added on 11/23/05.
+# Queue/RED/use_mark_p_ was changed to true on 12/22/05.
 Queue/RED set setbit_ false
 Queue/RED set gentle_ true
 ### Queue/RED/gentle_ was changed from false to true on Apr 23, 2001.
@@ -502,6 +508,10 @@ Application/SctpApp1 set reliability_ 0
 RandomVariable/Uniform set min_ 0.0
 RandomVariable/Uniform set max_ 1.0
 RandomVariable/Exponential set avg_ 1.0
+RandomVariable/Erlang set lambda_ 1.0
+RandomVariable/Erlang set k_ 1.0
+RandomVariable/Gamma set alpha_ 1.0
+RandomVariable/Gamma set beta_ 1.0
 RandomVariable/Pareto set avg_ 1.0
 RandomVariable/Pareto set shape_ 1.5
 RandomVariable/ParetoII set avg_ 10.0
@@ -517,6 +527,8 @@ RandomVariable/Normal set avg_ 0.0
 RandomVariable/Normal set std_ 1.0
 RandomVariable/LogNormal set avg_ 1.0
 RandomVariable/LogNormal set std_ 1.0
+RandomVariable/Weibull set scale_ 1.0
+RandomVariable/Weibull set shape_ 1.0
 
 ADC/MS set debug_ false
 ADC/HB set debug_ false
@@ -601,10 +613,14 @@ Simulator set McastAddr_ 0x80000000
 Simulator set AgentTrace_ ON
 Simulator set RouterTrace_ OFF
 Simulator set MacTrace_   OFF
+Simulator set PhyTrace_   OFF
 
 # use tagged traces or positional traces?
 Simulator set TaggedTrace_ OFF
 
+# this can be set to use custom Routing Agents implemented within dynamic libraries
+Simulator set rtAgentFunction_ ""
+
 SessionHelper set rc_ 0                      ;# just to eliminate warnings
 SessionHelper set debug_ false
 
@@ -659,6 +675,8 @@ LL set avoidReordering_ false ;	#not used
 
 Snoop set debug_ false
 
+
+
 #change wrt Mike's code
  # 802.11 MIB parameters
  #
@@ -674,6 +692,33 @@ Snoop set debug_ false
  Mac/802_11 set ShortRetryLimit_       7               ;# retransmittions
  Mac/802_11 set LongRetryLimit_        4               ;# retransmissions
 
+Mac/802_11 set bugFix_timer_ true;         # fix for when RTS/CTS not used
+# details at http://www.dei.unipd.it/wdyn/?IDsezione=2435
+
+ Mac/802_11 set BeaconInterval_	       0.1		;# 100ms	
+ Mac/802_11 set ScanType_	PASSIVE
+ Mac/802_11 set ProbeDelay_	0.0001		;# 0.1 ms
+ Mac/802_11 set MaxChannelTime_ 0.011		;# 11 ms
+ Mac/802_11 set MinChannelTime_ 0.005		; # 5 ms
+ Mac/802_11 set ChannelTime_ 0.12		;# 120 ms
+
+
+
+Mac/802_11Ext set HeaderDuration_   0.000020  ;# (SERVICE) 16bits last of PLCP header are not included
+Mac/802_11Ext set SymbolDuration_   0.000004  ;# (SERVICE) 16bits last of PLCP header are not included
+Mac/802_11Ext set BasicModulationScheme_ 0    ;# BPSK and coding rate 1/2 is the basic modulation scheme for header and ctrl packets
+Mac/802_11Ext set use_802_11a_flag_	true
+
+Mac/802_11Ext set CWMin_            15
+Mac/802_11Ext set CWMax_            1023
+Mac/802_11Ext set SlotTime_         0.000009
+Mac/802_11Ext set SIFS_             0.000016
+
+Mac/802_11Ext set RTSThreshold_     3000
+Mac/802_11Ext set ShortRetryLimit_  7
+Mac/802_11Ext set LongRetryLimit_   4
+
+Mac/802_11Ext set MAC_DBG           0
 
 
 # 
@@ -717,6 +762,23 @@ Phy/WirelessPhy set Pt_ 0.28183815
 Phy/WirelessPhy set freq_ 914e+6
 Phy/WirelessPhy set L_ 1.0  
 
+Phy/WirelessPhyExt set CSThresh_ 6.30957e-12           ;# -82 dBm
+Phy/WirelessPhyExt set noise_floor_ 7.96159e-14        ;# -101 dBm
+Phy/WirelessPhyExt set PowerMonitorThresh_ 2.653e-14   ;# -105.7 dBm (noise_floor_ / 3)
+Phy/WirelessPhyExt set Pt_  0.1
+Phy/WirelessPhyExt set freq_ 5.18e+9                   ;# 5.18 GHz
+Phy/WirelessPhyExt set HeaderDuration_   0.000020      ;# 20 us
+Phy/WirelessPhyExt set BasicModulationScheme_ 0        ;# BPSK
+Phy/WirelessPhyExt set L_ 1.0                          ;# default
+Phy/WirelessPhyExt set PreambleCaptureSwitch_ 1
+Phy/WirelessPhyExt set DataCaptureSwitch_ 0
+Phy/WirelessPhyExt set SINR_PreambleCapture_ 3.1623;   ;# 5 dB
+Phy/WirelessPhyExt set SINR_DataCapture_ 10.0;         ;# 10 dB
+Phy/WirelessPhyExt set trace_dist_ 1e6                 ;# trace until distance of 1 Mio. km ("infinty")
+Phy/WirelessPhyExt set PHY_DBG_ 0
+Phy/WirelessPhyExt set CPThresh_ 0 ;# not used by WirelessPhyExt, but available to be compabile with WirelessPhy
+Phy/WirelessPhyExt set RXThresh_ 0 ;# not used by WirelessPhyExt, but available to be compabile with WirelessPhy
+
 Phy/WiredPhy set bandwidth_ 10e6
 
 # Shadowing propagation model
@@ -725,6 +787,22 @@ Propagation/Shadowing set std_db_ 4.0
 Propagation/Shadowing set dist0_ 1.0
 Propagation/Shadowing set seed_ 0
 
+Propagation/Nakagami set gamma0_ 1.9
+Propagation/Nakagami set gamma1_ 3.8
+Propagation/Nakagami set gamma2_ 3.8
+
+Propagation/Nakagami set d0_gamma_ 200
+Propagation/Nakagami set d1_gamma_ 500
+
+Propagation/Nakagami set use_nakagami_dist_ false
+
+Propagation/Nakagami set m0_  1.5
+Propagation/Nakagami set m1_  0.75
+Propagation/Nakagami set m2_  0.75
+
+Propagation/Nakagami set d0_m_ 80
+Propagation/Nakagami set d1_m_ 200
+
 # Turning on/off sleep-wakeup cycles for SMAC
 Mac/SMAC set syncFlag_ 0
 
@@ -761,7 +839,6 @@ Agent/SCTP set associationMaxRetrans_ 10;# 10 attempts
 Agent/SCTP set pathMaxRetrans_ 5        ;# 5 attempts (per destination)
 Agent/SCTP set changePrimaryThresh_ -1  ;# infinite (ie, never change primary
 Agent/SCTP set maxInitRetransmits_ 8    ;# 8 attempts
-Agent/SCTP set oneHeartbeatTimer_ 1     ;# single heartbeat timer for all dests
 Agent/SCTP set heartbeatInterval_ 30    ;# 30 secs
 Agent/SCTP set mtu_ 1500                ;# MTU of ethernet (most common)
 Agent/SCTP set initialRwnd_ 65536       ;# default inital receiver window
@@ -795,6 +872,7 @@ Agent/SCTP set trace_all_ 0             ;# trace all vars ?
 ## These variables are set because they have to be bound to be traceable.
 ## This default does not matter to us at all.
 Agent/SCTP set cwnd_ 0                 ; 
+Agent/SCTP set rwnd_ 0                 ; 
 Agent/SCTP set rto_ 0                  ;
 Agent/SCTP set errorCount_ 0           ;
 Agent/SCTP set frCount_ 0              ;                                      
@@ -804,6 +882,19 @@ Agent/SCTP set rcdCount_ 0             ;# total count of route calc delays
 Agent/SCTP/MultipleFastRtx set mfrCount_ 0                                    
 Agent/SCTP/MfrTimestamp set mfrCount_ 0    
 
+## CMT variables
+Agent/SCTP/CMT set useCmtReordering_ 1  ;# Turn ON CMT Reordering algo
+Agent/SCTP/CMT set useCmtCwnd_ 1        ;# Turn ON CMT cwnd growth algo
+Agent/SCTP/CMT set useCmtDelAck_ 1      ;# Turn ON CMT delayed ack algo
+Agent/SCTP/CMT set eCmtRtxPolicy_ 4     ;# Default policy = RTX_CWND
+## CMT-PF variables
+Agent/SCTP/CMT set useCmtPF_ 0          ;# CMT-PF turned off
+Agent/SCTP/CMT set cmtPFCwnd_ 2	        ;# Cwnd in MTUs after HB-ACK (1 or 2)
+## CMT-PF trace variables
+Agent/SCTP/CMT set countPFToActiveNewData_ 0 ;# count of PF->Active changes
+                                              # for new data transfer
+Agent/SCTP/CMT set countPFToActiveRtxms_ 0;   # count of PF->Active changes
+                                              # for retransmissions
 
 Agent/TCP set seqno_ 0
 Agent/TCP set t_seqno_ 0
@@ -812,18 +903,21 @@ Agent/TCP set aggressive_maxburst_ 1 ;  # Added 2003/6/2.
 					# No change to default behavior.
 Agent/TCP set maxcwnd_ 0
 Agent/TCP set numdupacks_ 3
-Agent/TCP set numdupacksFrac_ 100 ;	# Added 2002/10/18.
-					# Set to 100000 for this to have
-					# no effect even for large windows.
+Agent/TCP set numdupacksFrac_ -1 ;	# Added 2002/10/18.
+					# Set to -1  for this to have
+					#   no effect even for large windows.
+					# Default changed to -1 on 2006/09/27.
+                                        # Set to 100 to have numdupacks_
+                                        #   up to 1/100-th of cwnd.
 Agent/TCP set exitFastRetrans_ true ;	# Added 2003/7/28.
 					# For clean exits of Fast Retransmit.
 					# False for old buggy behavior.
 Agent/TCP set window_ 20
-# Agent/TCP set windowInit_ 1
 Agent/TCP set windowInit_ 2 ;		# default changed on 2001/5/26.
 Agent/TCP set windowInitOption_ 1
-# Agent/TCP set syn_ false
 Agent/TCP set syn_ true ;		# default changed on 2001/5/17.
+Agent/TCP set max_connects_ -1 ;	# Variable added on 2007/9/25.
+					# Set to -1 for a no-op, 2007/9/28.
 Agent/TCP set windowOption_ 1
 Agent/TCP set windowConstant_ 4
 Agent/TCP set windowThresh_ 0.002
@@ -834,14 +928,15 @@ Agent/TCP set l_parameter_ 1.0 ;  	# for binomial congestion control
 Agent/TCP set overhead_ 0
 Agent/TCP set ecn_ 0
 Agent/TCP set old_ecn_ 0
+Agent/TCP set bugfix_ss_ 1 ;		# Variable added on 2006/06/13
 Agent/TCP set packetSize_ 1000
 Agent/TCP set tcpip_base_hdr_size_ 40
 Agent/TCP set ts_option_size_ 10; 	# in bytes
 Agent/TCP set bugFix_ true
 Agent/TCP set bugFix_ack_ false ;       # Variable added on 2003/08/13
-					# To allow some multiple Fast Retransmits
+					# To allow multiple Fast Retransmits
 Agent/TCP set bugFix_ts_ false ;	# Variable added on 2003/08/13
-					# To allow some multiple Fast Retransmits
+					# To allow multiple Fast Retransmits
 Agent/TCP set lessCareful_ false ;	# for the Less Careful variant of
 					# bugFix_, just for illustration.
 Agent/TCP set timestamps_ false
@@ -850,27 +945,23 @@ Agent/TCP set ts_resetRTO_ false ;	# Added 2003/07/24.
 					#   after any valid RTT measurement.
 Agent/TCP set slow_start_restart_ true
 Agent/TCP set restart_bugfix_ true
-# Agent/TCP set tcpTick_ 0.1
 Agent/TCP set tcpTick_ 0.01 ;		# default changed on 2002/03/07
 					# to reflect a changing reality.
-Agent/TCP set maxrto_ 100000
-#Agent/TCP set minrto_ 0 ;		# Variable added on 2001/05/10
-#Agent/TCP set minrto_ 1 ;              # Default set to 1 on 2001/05/14.
+Agent/TCP set maxrto_ 60 ; 		# default changed on 2007/03/28
+					#  to reflect RFC2988.
 Agent/TCP set minrto_ 0.2 ;		# Default changed to 200ms on 
 					#  2004/10/14, to match values
 					#  used by many implementations.
-					# Default set to 1 on 2001/05/14.
-					# Set to "0" to give old behavior.
 Agent/TCP set srtt_init_ 0
 Agent/TCP set rttvar_init_ 12
-Agent/TCP set rtxcur_init_ 6.0
+Agent/TCP set rtxcur_init_ 3.0 ;	# Default changed on 2006/01/21		
 Agent/TCP set T_SRTT_BITS 3
 Agent/TCP set T_RTTVAR_BITS 2
 Agent/TCP set rttvar_exp_ 2
+Agent/TCP set updated_rttvar_ true ;	# Variable added on 2006/1/21
 Agent/TCP set timerfix_ true ; 		# Variable added on 2001/05/11
  					# Set to "false" to give the old 
 					#  behavior. 
-# Agent/TCP set rfc2988_ false ;	# Variable added on 2002/03/07.
 Agent/TCP set rfc2988_ true ;		# Default set to "true" on 2002/03/07.
 					# Set rfc2988_ "true" to give RFC2988-
 					#  compliant behavior for timers.
@@ -889,32 +980,27 @@ Agent/TCP set srtt_ 0
 Agent/TCP set rttvar_ 0
 Agent/TCP set backoff_ 0
 Agent/TCP set maxseq_ 0
-# Agent/TCP set singledup_ 0
 Agent/TCP set singledup_ 1 ;		# default changed on 2001/11/28.
 Agent/TCP set LimTransmitFix_ false ;	# added on 2003/03/31.
-#Agent/TCP set LimTransmitFix_ true 
-Agent/TCP set precisionReduce_ false
+Agent/TCP set precisionReduce_ true ;	# default changed on 2006/1/24.
 Agent/TCP set oldCode_ false
 Agent/TCP set useHeaders_ true ;	# default changed on 2001/11/28. 
 
-# These are all variables for experimental high-speed TCP.
-# Agent/TCP set low_window_ 31
-Agent/TCP set low_window_ 38
-# low_window_ was changed on 2002/8/12 from 31 to 38, for low_p_ of 0.001.
-# This should be an undetectable change in performance.
+# These are all used for high-speed TCP.
+Agent/TCP set low_window_ 38 ;		# default changed on 2002/8/12.		
 Agent/TCP set high_window_ 83000
 Agent/TCP set high_p_ 0.0000001
 Agent/TCP set high_decrease_ 0.1
 Agent/TCP set max_ssthresh_ 0
 Agent/TCP set cwnd_range_ 0 ;		# cwnd_frac_ deleted on 6/6/04,
 					# replaced by cwnd_range_
-
 # For Quick-Start:
 Agent/TCP set rate_request_ 0
 Agent/TCP set qs_enabled_ false
 Agent/TCP set tcp_qs_recovery_ true ;	# Added on 2004/09/21, by Pasi.
 Agent/TCP set qs_request_mode_ 1
 Agent/TCP set qs_rtt_ 50
+Agent/TCP set print_request_ false
 Agent/TCP set qs_thresh_ 4
 
 # These are all variables for tracing information.
@@ -934,7 +1020,8 @@ Agent/TCP set QOption_ 0
 Agent/TCP set EnblRTTCtr_ 0
 Agent/TCP set control_increase_ 0
 
-Agent/TCP set SetCWRonRetransmit_ false ; # added on 2005/06/19.
+Agent/TCP set SetCWRonRetransmit_ true ; # added on 2005/06/19.
+				 	 # default changed on 2008/06/05. 
 
 # XXX Generate nam trace or plain old text trace for variables. 
 # When it's true, generate nam trace.
@@ -944,8 +1031,7 @@ Agent/TCP/Fack set ss-div4_ false
 Agent/TCP/Fack set rampdown_ false
 
 Agent/TCP/Reno/XCP set timestamps_ true
-Agent/TCP/Reno/XCP set xcp_sparse_ false
-Agent/TCP/Reno/XCP set tcpTick_	0.01
+Agent/TCP/FullTcp/Newreno/XCP set timestamps_ true
 
 Agent/TCP set eln_ 0
 Agent/TCP set eln_rxmit_thresh_ 1
@@ -970,12 +1056,7 @@ Agent/TCPSink set generateDSacks_ false
 Agent/TCPSink set qs_enabled_ false
 Agent/TCPSink set RFC2581_immediate_ack_ true
 Agent/TCPSink set bytes_ 0
-
-Agent/XCPSink set packetSize_ 40
-Agent/XCPSink set ts_echo_bugfix_ true
-Agent/XCPSink set bytes_ 0
-Agent/XCPSink set RFC2581_immediate_ack_ true
-Agent/XCPSink set ts_echo_rfc1323_ false
+Agent/TCPSink set ecn_syn_ false ;	# Added 2005/11/21 for SYN/ACK pkts.
 
 Agent/TCPSink/DelAck set interval_ 100ms
 catch {
@@ -1033,41 +1114,31 @@ Agent/TCP/RFC793edu set add793exponinc_    false
 
 Agent/TCP/FullTcp instproc done_data {} { }
 
-Agent/TFRC set packetSize_ 1000 
+# Dynamic state:
 Agent/TFRC set rate_ 0 
-Agent/TFRC set df_ 0.95 ;	# decay factor for accurate RTT estimate
-Agent/TFRC set tcp_tick_ 0.1 ;	
 Agent/TFRC set ndatapack_ 0 ;	# Number of packets sent
 Agent/TFRC set ndatabytes_ 0 ;	# Number of bytes sent
+Agent/TFRC set true_loss_rate_ 0.0 ; # For statistics only.
+# RTT:
 Agent/TFRC set srtt_init_ 0 ;	# Variables for tracking RTT	
 Agent/TFRC set rttvar_init_ 12  
 Agent/TFRC set rtxcur_init_ 6.0	
 Agent/TFRC set rttvar_exp_ 2	
 Agent/TFRC set T_SRTT_BITS 3	
 Agent/TFRC set T_RTTVAR_BITS 2	
-Agent/TFRC set InitRate_ 300 ;	# Initial send rate	
-Agent/TFRC set overhead_ 0 ;	# If > 0, dither outgoing packets
-Agent/TFRC set ssmult_ 2 ; 	# Rate of increase during slow-start:
-Agent/TFRC set bval_ 1 ;	# Value of B for TCP formula
-Agent/TFRC set ca_ 1 ; 	 	# Enable Sqrt(RTT) congestion avoidance
-Agent/TFRC set printStatus_ 0 
-Agent/TFRC set maxHeavyRounds_ 1; # Number of rounds for sending rate allowed
-				  #  to be greater than twice receiving rate.
-Agent/TFRC set conservative_ 0 ;  # Set to true for a conservative 
-				  # response to heavy congestion.
-Agent/TFRC set scmult_ 1.5 ;	# self clocking parameter for conservative_
-Agent/TFRC set ecn_ 0 ;		# Set to 1 for ECN-capable connection.
-Agent/TFRC set minrto_ 0.0 ;	# Minimum RTO, for use in TCP equation.
-				# The default is not to use minrto_.
-Agent/TFRC set SndrType_ 0 ;    # Set to 1 to use data-producing applications
-                                #   such as FTP.
-Agent/TFRC set oldCode_ false ; # Set to 1 to use old code for datalimited
-				#   applications.
-				# Parameter added on 12/18/02.
-Agent/TFRC set maxqueue_ MAXSEQ ;  # queue from application.
-Agent/TFRC set rate_init_ 2 ;		# Added on 10/20/2004
-				# Set to 1 for backward compatibility. 
-				# Default changed on 10/21/2004.
+# VoIP mode:
+Agent/TFRC set voip_ 0 ;        # Added on 10/23/2004      
+				# 1 for voip mode.
+Agent/TFRC set voip_max_pkt_rate_ 100 ;  # Max rate in pps, for voip mode.
+Agent/TFRC set fsize_ 1460 ;	# Default size for large TCP packets. 
+				# Used for VoIP mode.
+Agent/TFRC set headersize_ 32 ; # Size for packet headers.
+# End of VoIP mode.
+# Variants in the TFRC algorithms:
+# Agent/TFRC set standard_ 0 ;	# Added on 4/19/2007
+				# Set to 1 for RFC 3448 algorithms.
+				# Set to 2 for RFC 4342 algorithms.
+				# Set to 3 for RFC 3448bis algorithms.
 Agent/TFRC set rate_init_option_ 2 ;	# Added on 10/20/2004
 				# Set to 1 for backward compatibility. 
 				# Set to 2 for RFC 3390 initial rates
@@ -1075,19 +1146,39 @@ Agent/TFRC set rate_init_option_ 2 ;	# Added on 10/20/2004
 Agent/TFRC set slow_increase_ 1 ;	# Added on 10/20//2004
 				# Set to 1 for gradual rate changes.  
 				# This also gives backward compatibility.
-Agent/TFRC set ss_changes_ 1 ;	# Added on 10/21//2004. 
-				# Set to 0 for backward compatibility. 
-				# Set to 1 for slow-start changes to 
-				#  accomodate higher initial rates.
-				# Default changed on 10/22/2004.
-Agent/TFRC set voip_ 0 ;        # Added on 10/23/2004      
-				# 1 for voip mode.
-Agent/TFRC set voip_max_pkt_rate_ 100 ;  # Max rate in pps, for voip mode.
-Agent/TFRC set fsize_ 1460 ;	# Default size for large TCP packets. 
-				# Used for VoIP mode.
-Agent/TFRC set headersize_ 32 ; # Size for packet headers.
+# Agent/TFRC set ss_changes_ 1 ;	# Deleted on 3/14//2006. 
+Agent/TFRC set maxHeavyRounds_ 0; # Number of rounds for sending rate allowed
+				  #  to be greater than twice receiving rate.
+				  # Default changed on 3/27/2007, to conform
+				  # to RFC3448 and CCID 3.
+Agent/TFRC set conservative_ 0 ;  # Set to true for a conservative 
+				  # response to heavy congestion.
+Agent/TFRC set scmult_ 1.5 ;	# self clocking parameter for conservative_
+Agent/TFRC set oldCode_ false ; # Set to 1 to use old code for datalimited
+				#   applications.
+				# Parameter added on 12/18/02.
+# End of Variands.
+# Parameters:
+Agent/TFRC set packetSize_ 1000 
+Agent/TFRC set df_ 0.95 ;	# decay factor for accurate RTT estimate
+Agent/TFRC set tcp_tick_ 0.1 ;	
+Agent/TFRC set InitRate_ 300 ;	# Initial send rate	
+Agent/TFRC set overhead_ 0 ;	# If > 0, dither outgoing packets
+Agent/TFRC set ssmult_ 2 ; 	# Rate of increase during slow-start:
+Agent/TFRC set bval_ 1 ;	# Value of B for TCP formula
+Agent/TFRC set ca_ 1 ; 	 	# Enable Sqrt(RTT) congestion avoidance
+Agent/TFRC set printStatus_ 0 
+Agent/TFRC set ecn_ 0 ;		# Set to 1 for ECN-capable connection.
+Agent/TFRC set minrto_ 0.0 ;	# Minimum RTO, for use in TCP equation.
+				# The default is not to use minrto_.
+Agent/TFRC set SndrType_ 0 ;    # Set to 1 to use data-producing applications
+                                #   such as FTP.
+Agent/TFRC set maxqueue_ MAXSEQ ;  # queue from application.
+Agent/TFRC set rate_init_ 2 ;		# Added on 10/20/2004
+				# Set to 1 for backward compatibility. 
+				# Default changed on 10/21/2004.
 Agent/TFRC set useHeaders_ true ;	# Added on 2005/06/24. 
-Agent/TFRC set true_loss_rate_ 0.0 ; # For statistics only.
+Agent/TFRC set idleFix_ true ;	# Added on 2006/03/12.
 
 Agent/TFRCSink set packetSize_ 40
 Agent/TFRCSink set InitHistorySize_ 100000
@@ -1095,10 +1186,12 @@ Agent/TFRCSink set NumFeedback_ 1
 Agent/TFRCSink set AdjustHistoryAfterSS_ 1
 Agent/TFRCSink set NumSamples_ -1
 Agent/TFRCSink set discount_ 1;	# History Discounting
+Agent/TFRCSink set minDiscountRatio_ 0.5; # Minimum for history discounting.
 Agent/TFRCSink set printLoss_ 0
 Agent/TFRCSink set smooth_ 1 ;	# smoother Average Loss Interval
 Agent/TFRCSink set ShortIntervals_ 0 ; #  For calculating loss event rates 
                         	# for short loss intervals differently
+Agent/TFRCSink set ShortRtts_ 2 ; # Max num of RTTs in a short interval.
 Agent/TFRCSink set minlc_ 4
 Agent/TFRCSink set algo_ 1 ;  	# 1: algo from sigcomm paper 2: ewma 
 				# 3: fixed window
@@ -1133,6 +1226,9 @@ if [TclObject is-class Agent/TCP/FullTcp] {
 	Agent/TCP/FullTcp set open_cwnd_on_pack_ true; # ^ win on partial acks?
 	Agent/TCP/FullTcp set halfclose_ false; # do simplex closes (shutdown)?
 	Agent/TCP/FullTcp set nopredict_ false; # disable header prediction code?
+        Agent/TCP/FullTcp set ecn_syn_ false; # Make SYN/ACK packet ECN-Capable?
+        Agent/TCP/FullTcp set ecn_syn_wait_ 0; # Wait after marked SYN/ACK? 
+        Agent/TCP/FullTcp set debug_ false;  # Added Sept. 16, 2007.
 
 	Agent/TCP/FullTcp/Newreno set recov_maxburst_ 2; # max burst dur recov
 
@@ -1337,12 +1433,6 @@ Agent/LMS/Receiver set packetSize_ $lsize
 
 # Following defaults defined for TCP Quick Start
 # http://www.icir.org/floyd/quickstart.html
-
-Agent/TCP/Newreno/QS set rbp_scale_ 0.75
-Agent/TCP/Newreno/QS set rbp_segs_actually_paced_ 0
-Agent/TCP/Newreno/QS set rbp_inter_pace_delay_ 0
-Agent/TCP/Newreno/QS set rate_request_ 128
-
 Agent/QSAgent set qs_enabled_ 1
 Agent/QSAgent set old_classifier_ 0
 Agent/QSAgent set state_delay_ 0.2 ;	# Changed from 0.25 to 0.2, 2/25/05.
@@ -1353,15 +1443,6 @@ Agent/QSAgent set mss_ [Agent/TCP set packetSize_]
 Agent/QSAgent set rate_function_ 2
 Agent/QSAgent set algorithm_ 3	; 	# Changed from 2 to 3, 2/25/05.
 
-Agent/TCPSink/QS set sport_        0
-Agent/TCPSink/QS set dport_        0         
-
-Agent/TCPSink/QS set packetSize_ 40
-Agent/TCPSink/QS set maxSackBlocks_ 3
-Agent/TCPSink/QS set ts_echo_bugfix_ false
-Agent/TCPSink/QS set generateDSacks_ false
-Agent/TCPSink/QS set RFC2581_immediate_ack_ true
-
 Queue set util_weight_ 0.8
 Queue set util_check_intv_ 0.2 ;	# Changed from 1 to 0.2, 2/25/05.
 Queue set util_records_ 5 ; 		# Changed from 0 to 5, 2/25/05.
@@ -1369,3 +1450,16 @@ Queue set util_records_ 5 ; 		# Changed from 0 to 5, 2/25/05.
 # Quick Start definitions end here
 
 Delayer set debug_ false
+
+Agent/TCP/Linux set rtxcur_init_ 3
+Agent/TCP/Linux set maxrto_ 120
+Agent/TCP/Linux set minrto_ 0.2
+Agent/TCP/Linux set ts_resetRTO_ true
+Agent/TCP/Linux set next_pkts_in_flight_ 0
+Agent/TCP/Linux set delay_growth_ false
+
+Agent/PBC set payloadSize 200
+Agent/PBC set periodicBroadcastInterval 1
+Agent/PBC set periodicBroadcastVariance 0.1
+Agent/PBC set modulationScheme 0
+
