@@ -32,7 +32,7 @@
 # SUCH DAMAGE.
 #
 
-# @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcl/lib/ns-lib.tcl,v 1.271 2005/10/09 19:32:07 tomh Exp $
+# @(#) $Header: /cvsroot/nsnam/ns-2/tcl/lib/ns-lib.tcl,v 1.277 2009/01/15 06:23:49 tom_henderson Exp $
 
 
 #
@@ -224,6 +224,8 @@ if {[ns-hasSTL] == 1} {
 source ns-nix.tcl
 source ../pgm/ns-pgm.tcl
 source ../rtglib/ns-rtProtoLS.tcl
+source ../delaybox/delaybox.tcl
+source ../packmime/packmime.tcl
 }
 
 source ns-qsnode.tcl
@@ -325,10 +327,11 @@ Simulator instproc dumper obj {
 #                  -idlePower      (in W)
 #
 #                  -sleepPower 	   (in W)
-#
+#		   -sleepTime	   (in sec indicating when the node can start sleeping)
 #                  -agentTrace  ON
 #                  -routerTrace ON 
 #                  -macTrace OFF 
+#                  -phyTrace OFF 
 #                  -toraDebug OFF                
 #                  -movementTrace OFF
 # change wrt Mike's code
@@ -358,6 +361,7 @@ Simulator instproc rxPower  {val} { $self set rxPower_  $val }
 Simulator instproc idlePower  {val} { $self set idlePower_  $val }
 #
 Simulator instproc sleepPower  {val} { $self set sleepPower_  $val }
+Simulator instproc sleepTime  {val} { $self set sleepTime_  $val }
 Simulator instproc transitionPower  {val} { $self set transitionPower_  $val }
 Simulator instproc transitionTime  {val} { $self set transitionTime_  $val }
 #
@@ -367,12 +371,18 @@ Simulator instproc FECProc  {val} { $self set FECProc_  $val }
 Simulator instproc agentTrace  {val} { $self set agentTrace_  $val }
 Simulator instproc routerTrace  {val} { $self set routerTrace_  $val }
 Simulator instproc macTrace  {val} { $self set macTrace_  $val }
+Simulator instproc phyTrace  {val} { $self set phyTrace_  $val }
 Simulator instproc movementTrace  {val} { $self set movementTrace_  $val }
 Simulator instproc toraDebug {val} {$self set toraDebug_ $val }
 Simulator instproc satNodeType {val} {$self set satNodeType_ $val}
 Simulator instproc downlinkBW {val} {$self set downlinkBW_ $val}
 Simulator instproc stopTime {val} {$self set stopTime_ $val}
 
+# This method is needed so that new Routing Agents can be implemented in a dynamic
+# library and used without having to modify Simulator::create-wireless-node
+Simulator instproc rtAgentFunction {val} {$self set rtAgentFunction_ $val}
+
+
 # change wrt Mike's code
 Simulator instproc eotTrace  {val} { $self set eotTrace_  $val }
 Simulator instproc diffusionFilter {val} {$self set diffFilter_ $val}
@@ -442,8 +452,13 @@ Simulator instproc node-config args {
         $self instvar addressType_  routingAgent_ propType_  macTrace_ \
 	    routerTrace_ agentTrace_ movementTrace_ channelType_ channel_ \
 	    chan topoInstance_ propInstance_ mobileIP_ \
-	    rxPower_ txPower_ idlePower_ sleepPower_ transitionPower_ \
-	    transitionTime_ satNodeType_ eotTrace_
+	    rxPower_ txPower_ idlePower_ sleepPower_ sleepTime_ transitionPower_ \
+	    transitionTime_ satNodeType_ eotTrace_ phyTrace_
+
+	if [info exists phyTrace_] {
+		Simulator set PhyTrace_ $phyTrace_
+	}
+
 
         if [info exists macTrace_] {
 		Simulator set MacTrace_ $macTrace_
@@ -590,8 +605,8 @@ Simulator instproc create-wireless-node args {
         $self instvar routingAgent_ wiredRouting_ propInstance_ llType_ \
 	    macType_ ifqType_ ifqlen_ phyType_ chan antType_ \
 	    energyModel_ initialEnergy_ txPower_ rxPower_ \
-	    idlePower_ sleepPower_ transitionPower_ transitionTime_ \
-	    topoInstance_ level1_ level2_ inerrProc_ outerrProc_ FECProc_
+	    idlePower_ sleepPower_ sleepTime_ transitionPower_ transitionTime_ \
+	    topoInstance_ level1_ level2_ inerrProc_ outerrProc_ FECProc_ rtAgentFunction_
 
 	Simulator set IMEPFlag_ OFF
 
@@ -602,51 +617,58 @@ Simulator instproc create-wireless-node args {
         if { [info exist wiredRouting_] && $wiredRouting_ == "ON" } {
 		$node base-station [AddrParams addr2id [$node node-addr]]
     	}
-        switch -exact $routingAgent_ {
-	    DSDV {
-		    set ragent [$self create-dsdv-agent $node]
-	    }
-	    DSR {
-		    $self at 0.0 "$node start-dsr"
-	    }
-	    AODV {
-		    set ragent [$self create-aodv-agent $node]
-	    }
-	    TORA {
-		    Simulator set IMEPFlag_ ON
-		    set ragent [$self create-tora-agent $node]
-	    }
-	    DIFFUSION/RATE {
-		    eval $node addr $args
-		    set ragent [$self create-diffusion-rate-agent $node]
-	    }
-	    DIFFUSION/PROB {
-		    eval $node addr $args
-		    set ragent [$self create-diffusion-probability-agent $node]
-	    }
-	    Directed_Diffusion {
-		    eval $node addr $args
-		    set ragent [$self create-core-diffusion-rtg-agent $node]
-	    }
-	    FLOODING {
-		    eval $node addr $args
-		    set ragent [$self create-flooding-agent $node]
-	    }
-	    OMNIMCAST {
-		    eval $node addr $args
-		    set ragent [$self create-omnimcast-agent $node]
-	    }
-	    DumbAgent {
-		    set ragent [$self create-dumb-agent $node]
-	    }
-	    ManualRtg {
-		    set ragent [$self create-manual-rtg-agent $node]
-	    }
-	    default {
-		    eval $node addr $args
-		    puts "Wrong node routing agent!"
-		    exit
-	    }
+        if {$rtAgentFunction_ != ""} {
+		set ragent [$self $rtAgentFunction_ $node]
+	} else {
+		switch -exact $routingAgent_ {
+		    DSDV {
+			    set ragent [$self create-dsdv-agent $node]
+		    }
+		    DSR {
+			    $self at 0.0 "$node start-dsr"
+		    }
+		    AODV {
+			    set ragent [$self create-aodv-agent $node]
+		    }
+		    AOMDV {
+			    set ragent [$self create-aomdv-agent $node]
+		    }
+		    TORA {
+			    Simulator set IMEPFlag_ ON
+			    set ragent [$self create-tora-agent $node]
+		    }
+		    DIFFUSION/RATE {
+			    eval $node addr $args
+			    set ragent [$self create-diffusion-rate-agent $node]
+		    }
+		    DIFFUSION/PROB {
+			    eval $node addr $args
+			    set ragent [$self create-diffusion-probability-agent $node]
+		    }
+		    Directed_Diffusion {
+			    eval $node addr $args
+			    set ragent [$self create-core-diffusion-rtg-agent $node]
+		    }
+		    FLOODING {
+			    eval $node addr $args
+			    set ragent [$self create-flooding-agent $node]
+		    }
+		    OMNIMCAST {
+			    eval $node addr $args
+			    set ragent [$self create-omnimcast-agent $node]
+		    }
+		    DumbAgent {
+			    set ragent [$self create-dumb-agent $node]
+		    }
+		    ManualRtg {
+			    set ragent [$self create-manual-rtg-agent $node]
+		    }
+		    default {
+			    eval $node addr $args
+			    puts "Wrong node routing agent!"
+			    exit
+		    }
+		}
 	}
 
 	# errProc_ and FECProc_ are an option unlike other 
@@ -732,6 +754,9 @@ Simulator instproc create-wireless-node args {
 	if [info exists sleepPower_] {
 		$node setPsleep $sleepPower_
         }
+	if [info exists sleepTime_] {
+		$node setTSleep $sleepTime_
+        }
 	if [info exists transitionPower_] {
 		$node setPtransition $transitionPower_
         }
@@ -828,6 +853,14 @@ Simulator instproc create-aodv-agent { node } {
         return $ragent
 }
 
+# AOMDV patch
+Simulator instproc create-aomdv-agent { node } {
+	set ragent [new Agent/AOMDV [$node node-addr]]
+	$self at 0.0 "$ragent start"
+	$node set ragent_ $ragent
+	return $ragent
+}
+
 Simulator instproc use-newtrace {} {
 	Simulator set WirelessNewTrace_ 1
 } 
@@ -1844,8 +1877,8 @@ Classifier instproc adjacents {} {
 Classifier instproc in-slot? slot {
 	$self instvar slots_
 	set ret ""
-	if {[array size slots_] < $slot} {
-		set ret slots_($slot)
+	if {[info exists slots_($slot)]} {
+	        set ret $slots_($slot)
 	}
 	set ret
 }
