@@ -49,6 +49,7 @@
  */
 
 /*
+ *
  * Full-TCP : A two-way TCP very similar to the 4.4BSD version of Reno TCP.
  * This version also includes variants Tahoe, NewReno, and SACK.
  *
@@ -108,7 +109,7 @@
 
 #ifndef lint
 static const char rcsid[] =
-    "@(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcp/tcp-full.cc,v 1.118 2005/08/02 04:02:58 tomh Exp $ (LBL)";
+    "@(#) $Header: /cvsroot/nsnam/ns-2/tcp/tcp-full.cc,v 1.128 2009/03/29 20:59:41 sallyfloyd Exp $ (LBL)";
 #endif
 
 #include "ip.h"
@@ -194,6 +195,9 @@ FullTcpAgent::delay_bind_init_all()
         delay_bind_init_one("open_cwnd_on_pack_");
         delay_bind_init_one("halfclose_");
         delay_bind_init_one("nopredict_");
+        delay_bind_init_one("ecn_syn_");
+        delay_bind_init_one("ecn_syn_wait_");
+        delay_bind_init_one("debug_");
         delay_bind_init_one("spa_thresh_");
 
 	TcpAgent::delay_bind_init_all();
@@ -222,6 +226,9 @@ FullTcpAgent::delay_bind_dispatch(const char *varName, const char *localName, Tc
         if (delay_bind_bool(varName, localName, "open_cwnd_on_pack_", &open_cwnd_on_pack_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "halfclose_", &halfclose_, tracer)) return TCL_OK;
         if (delay_bind_bool(varName, localName, "nopredict_", &nopredict_, tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "ecn_syn_", &ecn_syn_, tracer)) return TCL_OK;
+        if (delay_bind(varName, localName, "ecn_syn_wait_", &ecn_syn_wait_, tracer)) return TCL_OK;
+        if (delay_bind_bool(varName, localName, "debug_", &debug_, tracer)) return TCL_OK;
 
         return TcpAgent::delay_bind_dispatch(varName, localName, tracer);
 }
@@ -369,9 +376,9 @@ FullTcpAgent::advance_bytes(int nb)
 		break;
 
 	default:
-	    fprintf(stderr,
-	    "%f: FullTcpAgent::advance(%s): cannot advance while in state %s\n",
-		 now(), name(), statestr(state_));
+            if (debug_) 
+	            fprintf(stderr, "%f: FullTcpAgent::advance(%s): cannot advance while in state %s\n",
+		         now(), name(), statestr(state_));
 
 	}
 
@@ -482,14 +489,16 @@ FullTcpAgent::usrclosed()
 	case TCPS_FIN_WAIT_2:
 	case TCPS_CLOSING:
 		/* usr asked for a close more than once [?] */
-		fprintf(stderr,
-		  "%f FullTcpAgent(%s): app close in bad state %s\n",
-		  now(), name(), statestr(state_));
+                if (debug_)
+		         fprintf(stderr,
+		           "%f FullTcpAgent(%s): app close in bad state %s\n",
+		           now(), name(), statestr(state_));
 		break;
 	default:
-		fprintf(stderr,
-		  "%f FullTcpAgent(%s): app close in unknown state %s\n",
-		  now(), name(), statestr(state_));
+                if (debug_)
+		        fprintf(stderr,
+		        "%f FullTcpAgent(%s): app close in unknown state %s\n",
+		        now(), name(), statestr(state_));
 	}
 
 	return;
@@ -618,6 +627,15 @@ FullTcpAgent::reset()
 		recent_ = recent_age_ = -1.0;
 
 	fastrecov_ = FALSE;
+
+	closed_ = 0;
+	close_on_empty_ = FALSE;
+
+        if (ecn_syn_)
+                ecn_syn_next_ = 1;
+        else
+                ecn_syn_next_ = 0;
+
 }
 
 /*
@@ -816,9 +834,9 @@ FullTcpAgent::ack_action(Packet* p)
  * Also, set the size of the tcp header.
  */
 void
-FullTcpAgent::sendpacket(int seqno, int ackno, int pflags, int datalen, int reason)
+FullTcpAgent::sendpacket(int seqno, int ackno, int pflags, int datalen, int reason, Packet *p)
 {
-        Packet* p = allocpkt();
+        if (!p) p = allocpkt();
         hdr_tcp *tcph = hdr_tcp::access(p);
 	hdr_flags *fh = hdr_flags::access(p);
 
@@ -843,15 +861,21 @@ FullTcpAgent::sendpacket(int seqno, int ackno, int pflags, int datalen, int reas
 	 *	ect_: I am doing ECN (ecn_ should be T and peer does ECN)
 	 */
 
-	// set ect on data packets (not syn or ack packets)
-	if ( datalen > 0 && ecn_ ){
+	if (datalen > 0 && ecn_ ){
+	        // set ect on data packets 
 		fh->ect() = ect_;	// on after mutual agreement on ECT
-	}
-	else {
+        } else if (ecn_ && ecn_syn_ && ecn_syn_next_ && (pflags & TH_SYN) && (pflags & TH_ACK)) {
+                // set ect on syn/ack packet, if syn packet was negotiating ECT
+               	fh->ect() = ect_;
+	} else {
 		/* Set ect() to 0.  -M. Weigle 1/19/05 */
 		fh->ect() = 0;
 	}
-
+	if (ecn_ && ect_ && recent_ce_ ) { 
+		// This is needed here for the ACK in a SYN, SYN/ACK, ACK
+		// sequence.
+		pflags |= TH_ECE;
+	}
         // fill in CWR and ECE bits which don't actually sit in
         // the tcp_flags but in hdr_flags
         if ( pflags & TH_ECE) {
@@ -981,7 +1005,6 @@ FullTcpAgent::foutput(int seqno, int reason)
 		datalen = maxseg_;
 	}
 
-
 	//
 	// this is an option that causes us to slow-start if we've
 	// been idle for a "long" time, where long means a rto or longer
@@ -1079,14 +1102,23 @@ send:
                 pflags &= ~TH_CWR;
         }
 	else if (ecn_ && ect_ && cong_action_ && 
-	             (!is_retransmit || SetCWRonRetransmit_)) 
+	             (!is_retransmit || SetCWRonRetransmit_)) {
 		/* 
 		 * Don't set CWR for a retranmitted SYN+ACK (has ecn_ 
-		 * and cong_action_ set) or on any retransmits.
+		 * and cong_action_ set).
 		 * -M. Weigle 6/19/02
+                 *
+                 * SetCWRonRetransmit_ was changed to true,
+                 * allowing CWR on retransmitted data packets.  
+                 * See test ecn_burstyEcn_reno_full 
+                 * in test-suite-ecn-full.tcl.
+		 * - Sally Floyd, 6/5/08.
 		 */
 		/* set CWR if necessary */
 		pflags |= TH_CWR;
+		/* Turn cong_action_ off: Added 6/5/08, Sally Floyd. */
+		cong_action_ = FALSE;
+	}
 
 	/* moved from sendpacket()  -M. Weigle 6/19/02 */
 	//
@@ -1099,7 +1131,9 @@ send:
 	}
   
         /* set ECE if necessary */
-        if (ecn_ && ect_ && recent_ce_ ) pflags |= TH_ECE;
+        if (ecn_ && ect_ && recent_ce_ ) {
+		pflags |= TH_ECE;
+	}
 
         /* 
          * Tack on the FIN flag to the data segment if close_on_empty_
@@ -1286,6 +1320,7 @@ FullTcpAgent::newack(Packet* pkt)
         /*
          * Update RTT only if it's OK to do so from info in the flags header.
          * This is needed for protocols in which intermediate agents
+
          * in the network intersperse acks (e.g., ack-reconstructors) for
          * various reasons (without violating e2e semantics).
          */
@@ -1310,7 +1345,6 @@ FullTcpAgent::newack(Packet* pkt)
 			rtt_active_ = FALSE;
 			rtt_update(now() - rtt_ts_);
                 }
-
 		if (!ect_ || !ecn_backoff_ ||
 		    !hdr_flags::access(pkt)->ecnecho()) {
 			/*
@@ -1321,6 +1355,7 @@ FullTcpAgent::newack(Packet* pkt)
 			t_backoff_ = 1;
 			ecn_backoff_ = 0;
 		}
+
         }
 	return;
 }
@@ -1507,9 +1542,11 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 	 */
 
 	if (state_ == TCPS_CLOSED) {
-		fprintf(stderr, "%f: FullTcp(%s): recv'd pkt in CLOSED state: ",
-			now(), name());
-		prpkt(pkt);
+                if (debug_) {
+		        fprintf(stderr, "%f: FullTcp(%s): recv'd pkt in CLOSED state: ",
+			    now(), name());
+		        prpkt(pkt);
+                }
 		goto drop;
 	}
 
@@ -1517,7 +1554,6 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
          * Process options if not in LISTEN state,
          * else do it below
          */
-
 	if (state_ != TCPS_LISTEN)
 		dooptions(pkt);
 
@@ -1528,30 +1564,16 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 	 * at time t0 = (0.0 + k * interval_) for some k such
 	 * that t0 > now
 	 */
-
 	if (delack_interval_ > 0.0 &&
 	    (delack_timer_.status() != TIMER_PENDING)) {
 		int last = int(now() / delack_interval_);
 		delack_timer_.resched(delack_interval_ * (last + 1.0) - now());
 	}
 
-	/*
-	 * sanity check for ECN: shouldn't be seeing a CE bit if
-	 * ECT wasn't set on the packet first.  If we see this, we
-	 * probably have a misbehaving router...
-	 */
-
-	if (fh->ce() && !fh->ect()) {
-	    fprintf(stderr,
-	    "%f: FullTcpAgent::recv(%s): warning: CE bit on, but ECT false!\n",
-		now(), name());
-	}
-
 	/*
 	 * Try header prediction: in seq data or in seq pure ACK
 	 *	with no funny business
 	 */
-
 	if (!nopredict_ && predict_ok(pkt)) {
                 /*
                  * If last ACK falls within this segment's sequence numbers,
@@ -1573,17 +1595,18 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 		// generate a stream of ecnecho bits until we see a true
 		// cong_action bit
 		//
-		if (ecn_) {
-			if (fh->ce() && fh->ect()) {
-				// no CWR from peer yet... arrange to
-				// keep sending ECNECHO
-				recent_ce_ = TRUE;
-			} else if (fh->cwr()) {
-				// got CWR response from peer.. stop
-				// sending ECNECHO bits
-				recent_ce_ = FALSE;
-			}
-		}
+
+	    	if (ecn_) {
+	    		if (fh->ce() && fh->ect()) {
+	    			// no CWR from peer yet... arrange to
+	    			// keep sending ECNECHO
+	    			recent_ce_ = TRUE;
+	    		} else if (fh->cwr()) {
+	    			// got CWR response from peer.. stop
+	    			// sending ECNECHO bits
+	    			recent_ce_ = FALSE;
+	    		}
+	    	}
 
 		// Header predication basically looks to see
 		// if the incoming packet is an expected pure ACK
@@ -1656,18 +1679,21 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 	case TCPS_LISTEN:	/* awaiting peer's SYN */
 
 		if (tiflags & TH_ACK) {
-		    	fprintf(stderr,
+                        if (debug_) {
+		    	        fprintf(stderr,
 		    		"%f: FullTcpAgent(%s): warning: recv'd ACK while in LISTEN: ",
-				now(), name());
-			prpkt(pkt);
+				    now(), name());
+			        prpkt(pkt);
+                        }
 			// don't want ACKs in LISTEN
 			goto dropwithreset;
 		}
 		if ((tiflags & TH_SYN) == 0) {
-		    	fprintf(stderr,
-		    		"%f: FullTcpAgent(%s): warning: recv'd NON-SYN while in LISTEN\n",
+                        if (debug_) {
+		    	        fprintf(stderr, "%f: FullTcpAgent(%s): warning: recv'd NON-SYN while in LISTEN\n",
 				now(), name());
-			prpkt(pkt);
+			        prpkt(pkt);
+                        }
 			// any non-SYN is discarded
 			goto drop;
 		}
@@ -1684,7 +1710,6 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 
 		// check for a ECN-SYN with ECE|CWR
 		if (ecn_ && fh->ecnecho() && fh->cong_action()) {
-
 			ect_ = TRUE;
 		}
 
@@ -1718,22 +1743,46 @@ FullTcpAgent::recv(Packet *pkt, Handler*)
 		if ((tiflags & TH_ACK) &&
 			((ackno <= iss_) || (ackno > maxseq_))) {
 			// not an ACK for our SYN, discard
-			fprintf(stderr,
-			    "%f: FullTcpAgent::recv(%s): bad ACK for our SYN: ",
+                        if (debug_) {
+			       fprintf(stderr, "%f: FullTcpAgent::recv(%s): bad ACK for our SYN: ",
 			        now(), name());
-			prpkt(pkt);
+			        prpkt(pkt);
+                        }
 			goto dropwithreset;
 		}
 
 		if ((tiflags & TH_SYN) == 0) {
-			fprintf(stderr,
-			    "%f: FullTcpAgent::recv(%s): no SYN for our SYN: ",
+                        if (debug_) {
+			        fprintf(stderr, "%f: FullTcpAgent::recv(%s): no SYN for our SYN: ",
 			        now(), name());
-			prpkt(pkt);
+			        prpkt(pkt);
+                        }
 			goto drop;
 		}
 
 		/* looks like an ok SYN or SYN+ACK */
+                // If ecn_syn_wait is set to 2:
+		// Check if CE-marked SYN/ACK packet, then just send an ACK
+                //  packet with ECE set, and drop the SYN/ACK packet.
+                //  Don't update TCP state. 
+		if (tiflags & TH_ACK) 
+		{
+                        if (ecn_ && fh->ecnecho() && !fh->cong_action() && ecn_syn_wait_ == 2) 
+                        // if SYN/ACK packet and ecn_syn_wait_ == 2
+			{
+	    		        if ( fh->ce() ) 
+                                // If SYN/ACK packet is CE-marked
+				{
+					//cancel_rtx_timer();
+					//newack(pkt);
+					set_rtx_timer();
+					sendpacket(t_seqno_, rcv_nxt_, TH_ACK|TH_ECE, 0, 0);
+					goto drop;
+				}
+	    		}
+		}
+
+
 #ifdef notdef
 cancel_rtx_timer();	// cancel timer on our 1st SYN [does this belong!?]
 #endif
@@ -1742,10 +1791,11 @@ cancel_rtx_timer();	// cancel timer on our 1st SYN [does this belong!?]
 
 		if (tiflags & TH_ACK) {
 			// SYN+ACK (our SYN was acked)
-			// CHECKME
-                        // Check ECN-SYN+ACK packet
-                        if (ecn_ && fh->ecnecho() && !fh->cong_action())
+                        if (ecn_ && fh->ecnecho() && !fh->cong_action()) {
                                 ect_ = TRUE;
+	    		        if ( fh->ce() ) 
+	    				recent_ce_ = TRUE;
+	    		}
 			highest_ack_ = ackno;
 			cwnd_ = initial_window();
 
@@ -1963,10 +2013,11 @@ trimthenstep6:
 	}
 
 	if (tiflags & TH_SYN) {
-		fprintf(stderr,
-		    "%f: FullTcpAgent::recv(%s) received unexpected SYN (state:%d): ",
+                if (debug_) {
+		        fprintf(stderr, "%f: FullTcpAgent::recv(%s) received unexpected SYN (state:%d): ",
 		        now(), name(), state_);
-		prpkt(pkt);
+		        prpkt(pkt);
+                }
 		goto dropwithreset;
 	}
 
@@ -1980,11 +2031,13 @@ trimthenstep6:
 		 * -M. Weigle 07/24/01
 		 */
 		if (state_ != TCPS_SYN_RECEIVED) {
-			fprintf(stderr, "%f: FullTcpAgent::recv(%s) got packet lacking ACK (state:%d): ",
+                        if (debug_) {
+			        fprintf(stderr, "%f: FullTcpAgent::recv(%s) got packet lacking ACK (state:%d): ",
 				now(), name(), state_);
-			prpkt(pkt);
-			goto drop;
+			        prpkt(pkt);
+                        }
 		}
+		goto drop;
 	}
 
 	/*
@@ -1995,12 +2048,41 @@ trimthenstep6:
 	case TCPS_SYN_RECEIVED:	/* want ACK for our SYN+ACK */
 		if (ackno < highest_ack_ || ackno > maxseq_) {
 			// not in useful range
-		    	fprintf(stderr,
-		    		"%f: FullTcpAgent(%s): ack(%d) not in range while in SYN_RECEIVED: ",
+                        if (debug_) {
+		    	        fprintf(stderr, "%f: FullTcpAgent(%s): ack(%d) not in range while in SYN_RECEIVED: ",
 			 	now(), name(), ackno);
-			prpkt(pkt);
+			        prpkt(pkt);
+                        }
 			goto dropwithreset;
 		}
+
+		if (ecn_ && ect_ && ecn_syn_ && fh->ecnecho() && ecn_syn_wait_ == 2) 
+		{
+		// The SYN/ACK packet was ECN-marked.
+		// Reset the rtx timer, send another SYN/ACK packet
+                //  immediately, and drop the ACK packet.
+                // Do not move to TCPS_ESTB state or update TCP variables.
+			cancel_rtx_timer();
+			ecn_syn_next_ = 0;
+			foutput(iss_, REASON_NORMAL);
+			wnd_init_option_ = 1;
+                        wnd_init_ = 1;
+			goto drop;
+		} 
+		if (ecn_ && ect_ && ecn_syn_ && fh->ecnecho() && ecn_syn_wait_ < 2) {
+		// The SYN/ACK packet was ECN-marked.
+			if (ecn_syn_wait_ == 1) {
+				// A timer will be called in ecn().
+				cwnd_ = 1;
+				use_rtt_ = 1; //KK, wait for timeout() period
+			} else {
+			        // Congestion window will be halved in ecn().
+				cwnd_ = 2;
+			}
+		} else  {
+			cwnd_ = initial_window();
+		}
+	
                 /*
                  * Make transitions:
                  *      SYN-RECEIVED  -> ESTABLISHED
@@ -2012,9 +2094,10 @@ trimthenstep6:
                 } else {
                         newstate(TCPS_ESTABLISHED);
                 }
-		cwnd_ = initial_window();
+
 		/* fall into ... */
 
+
         /*
          * In ESTABLISHED state: drop duplicate ACKs; ACK out of range
          * ACKs.  If the ack is in the range
@@ -2050,7 +2133,7 @@ trimthenstep6:
                 if (ecn_) {
                         if (fh->ce() && fh->ect())
                                 recent_ce_ = TRUE;
-                        else if (fh->cwr())
+                        else if (fh->cwr()) 
                                 recent_ce_ = FALSE;
                 }
 
@@ -2104,7 +2187,6 @@ trimthenstep6:
 				} else if (++dupacks_ == tcprexmtthresh_) {
 					// ACK at highest_ack_ AND meets threshold
 					//trace_event("FAST_RECOVERY");
-
 					dupack_action(); // maybe fast rexmt
 					goto drop;
 
@@ -2139,10 +2221,11 @@ process_ACK:
 
 		if (ackno > maxseq_) {
 			// ack more than we sent(!?)
-			fprintf(stderr,
-			    "%f: FullTcpAgent::recv(%s) too-big ACK (maxseq:%d): ",
+                        if (debug_) {
+			        fprintf(stderr, "%f: FullTcpAgent::recv(%s) too-big ACK (maxseq:%d): ",
 				now(), name(), int(maxseq_));
-			prpkt(pkt);
+			        prpkt(pkt);
+                        }
 			goto dropafterack;
 		}
 
@@ -2179,7 +2262,6 @@ process_ACK:
 		 * if this is an ACK with an ECN indication, handle this
 		 * but not if it is a syn packet
 		 */
-
 		if (fh->ecnecho() && !(tiflags&TH_SYN) )
 		if (fh->ecnecho()) {
 			// Note from Sally: In one-way TCP,
@@ -2214,9 +2296,10 @@ process_ACK:
                  */
 		if ((!delay_growth_ || (rcv_nxt_ > 0)) &&
 		    last_state_ == TCPS_ESTABLISHED) {
-			if (!partial || open_cwnd_on_pack_)
-			  if (!ect_ || !hdr_flags::access(pkt)->ecnecho())
+			if (!partial || open_cwnd_on_pack_) {
+                           if (!ect_ || !hdr_flags::access(pkt)->ecnecho())
 				opencwnd();
+                        }
 		}
 
 		if ((state_ >= TCPS_FIN_WAIT_1) && (ackno == maxseq_)) {
@@ -2270,10 +2353,11 @@ process_ACK:
 				goto drop;
 			} else {
 				// should be a FIN we've seen
-                                fprintf(stderr,
-                                "%f: FullTcpAgent(%s)::received non-ACK (state:%d): ",
+                                if (debug_) {
+                                        fprintf(stderr, "%f: FullTcpAgent(%s)::received non-ACK (state:%d): ",
                                         now(), name(), state_);
-				prpkt(pkt);
+				        prpkt(pkt);
+                                }
                         }
 			break;
 
@@ -2440,6 +2524,7 @@ drop:
    	Packet::free(pkt);
 	return;
 }
+
 /*  
  * Dupack-action: what to do on a DUP ACK.  After the initial check
  * of 'recover' below, this function implements the following truth
@@ -2466,10 +2551,9 @@ FullTcpAgent::dupack_action()
 	fastrecov_ = TRUE;
 	rtxbytes_ = 0;
 
-        if (recovered || (!bug_fix_ && !ecn_) ||
-	    // Q: is last_cwnd_action == dupack enough to trigger?
-	    // probably not.  may need to fix 1-way reno model as well
-	    last_cwnd_action_ == CWND_ACTION_DUPACK) {
+        if (recovered || (!bug_fix_ && !ecn_) 
+            || (last_cwnd_action_ == CWND_ACTION_DUPACK)
+            || ( highest_ack_ == 0)) {
                 goto full_reno_action;
         }       
     
@@ -2509,9 +2593,10 @@ FullTcpAgent::timeout_action()
 	recover_ = maxseq_;
 
 	if (cwnd_ < 1.0) {
-	    fprintf(stderr,
-	    "%f: FullTcpAgent(%s):: resetting cwnd from %f to 1\n",
-		 now(), name(), double(cwnd_));
+                if (debug_) {
+	            fprintf(stderr, "%f: FullTcpAgent(%s):: resetting cwnd from %f to 1\n",
+		    now(), name(), double(cwnd_));
+                }
 		cwnd_ = 1.0;
 	}
 
@@ -2552,8 +2637,10 @@ FullTcpAgent::timeout(int tno)
 	 */
 	if (state_ == TCPS_LISTEN) {
 	 	// shouldn't be getting timeouts here
-		fprintf(stderr, "%f: FullTcpAgent(%s): unexpected timeout %d in state %s\n",
+                if (debug_) {
+		        fprintf(stderr, "%f: FullTcpAgent(%s): unexpected timeout %d in state %s\n",
 			now(), name(), tno, statestr(state_));
+                }
 		return;
 	}
 
@@ -2656,9 +2743,7 @@ TahoeFullTcpAgent::dupack_action()
 	fastrecov_ = TRUE;
 	rtxbytes_ = 0;
 
-        if (recovered || (!bug_fix_ && !ecn_) ||
-	    // Q: is last_cwnd_action dupack enough?
-            last_cwnd_action_ == CWND_ACTION_DUPACK) {
+        if (recovered || (!bug_fix_ && !ecn_) || highest_ack_ == 0) {
                 goto full_tahoe_action;
         }
    
