@@ -30,7 +30,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/tcp/tcp.h,v 1.121 2005/06/20 16:30:30 sfloyd Exp $ (LBL)
+ * @(#) $Header: /cvsroot/nsnam/ns-2/tcp/tcp.h,v 1.130 2007/09/29 01:07:22 sallyfloyd Exp $ (LBL)
  */
 #ifndef ns_tcp_h
 #define ns_tcp_h
@@ -172,9 +172,10 @@ struct hstcp {
 };
 
 class TcpAgent : public Agent {
+	friend class XcpEndsys;
 public:
 	TcpAgent();
-	~TcpAgent() {free(tss);}
+	virtual ~TcpAgent() {free(tss);}
         virtual void recv(Packet*, Handler*);
 	virtual void timeout(int tno);
 	virtual void timeout_nonrtx(int tno);
@@ -194,7 +195,70 @@ protected:
 	virtual void delay_bind_init_all();
 	virtual int delay_bind_dispatch(const char *varName, const char *localName, TclObject *tracer);
 
+	double boot_time_;	/* where between 'ticks' this sytem came up */
+	double overhead_;
+	double wnd_;
+	double wnd_const_;
+	double wnd_th_;		/* window "threshold" */
+	double wnd_init_;
+	double wnd_restart_;
+	double tcp_tick_;	/* clock granularity */
+	int wnd_option_;
+	int wnd_init_option_;   /* 1 for using wnd_init_ */
+				/* 2 for using large initial windows */
+	double decrease_num_;   /* factor for multiplicative decrease */
+	double increase_num_;   /* factor for additive increase */
+	int tcpip_base_hdr_size_;  /* size of base TCP/IP header */
+	int maxcwnd_;		/* max # cwnd can ever be */
+        int numdupacks_;	/* dup ACKs before fast retransmit */
+	int numdupacksFrac_;	/* for a larger numdupacks_ with large */
+				/* windows */
+
+	/* connection and packet dynamics */
+	virtual void output(int seqno, int reason = 0);
+	virtual void send_much(int force, int reason, int maxburst = 0);
+	virtual void newtimer(Packet*);
+	virtual void dupack_action();		/* do this on dupacks */
+	virtual void send_one();		/* do this on 1-2 dupacks */
+	virtual void opencwnd();
+
+	void slowdown(int how);			/* reduce cwnd/ssthresh */
+	void ecn(int seqno);		/* react to quench */
+	virtual void set_initial_window();	/* set IW */
+	double initial_window();		/* what is IW? */
+	void reset();
+	void newack(Packet*);
+	void finish(); /* called when the connection is terminated */
+	int network_limited();	/* Sending limited by network? */
+	double limited_slow_start(double cwnd, int max_ssthresh, double increment);
+				/* Limited slow-start for high windows */
+	virtual int numdupacks(double cwnd); 	/* for getting numdupacks_ */
+	/* End of section of connection and packet dynamics.  */
+
+	/* General dynamic state. */
 	TracedInt t_seqno_;	/* sequence number */
+	TracedInt dupacks_;	/* number of duplicate acks */
+	TracedInt curseq_;	/* highest seqno "produced by app" */
+	TracedInt highest_ack_;	/* not frozen during Fast Recovery */
+	TracedDouble cwnd_;	/* current window */
+	TracedInt ssthresh_;	/* slow start threshold */
+	TracedInt maxseq_;	/* used for Karn algorithm */
+				/* highest seqno sent so far */
+	int last_ack_;		/* largest consecutive ACK, frozen during
+				 *		Fast Recovery */
+	int recover_;		/* highest pkt sent before dup acks, */
+				/*   timeout, or source quench/ecn */
+	int last_cwnd_action_;	/* CWND_ACTION_{TIMEOUT,DUPACK,ECN} */
+	int count_;		/* used in window increment algorithms */
+	int rtt_active_;	/* 1 if a rtt sample is pending */
+	int rtt_seq_;		/* seq # of timed seg if rtt_active_ is 1 */
+	double rtt_ts_;		/* time at which rtt_seq_ was sent */
+	double firstsent_;	/* When first packet was sent  --Allman */
+	double lastreset_;	/* W.N. Last time connection was reset - for */
+				/* detecting pkts from previous incarnations */
+	int closed_;            /* whether this connection has closed */
+	/* End of general dynamic state. */
+
 	/*
 	 * State encompassing the round-trip-time estimate.
 	 * srtt and rttvar are stored as fixed point;
@@ -203,7 +267,8 @@ protected:
 	TracedInt t_rtt_;      	/* round trip time */
 	TracedInt t_srtt_;     	/* smoothed round-trip time */
 	TracedInt t_rttvar_;   	/* variance in round-trip time */
-	TracedInt t_backoff_;	/* current multiplier, 1 if not backed off */
+	TracedInt t_backoff_;	/* current multiplier of RTO, */
+				/*   1 if not backed off */
 	#define T_RTT_BITS 0
 	int T_SRTT_BITS;        /* exponent of weight for updating t_srtt_ */
 	int srtt_init_;		/* initial value for computing t_srtt_ */
@@ -218,6 +283,16 @@ protected:
 	virtual void rtt_backoff();		/* double multiplier */
 	/* End of state for the round-trip-time estimate. */
 
+        /* RTOs: */
+	double maxrto_;		/* max value of an RTO */
+	double minrto_;         /* min value of an RTO */
+	int ts_resetRTO_;	/* Un-backoff RTO after any valid RTT, */
+				/*   including from a retransmitted pkt?  */
+				/* The old version was "false". */
+				/* But "true" gives better performance, and */
+                                /* seems conformant with RFC 2988. */
+        /* End of section for RTOs. */
+
 	/* Timestamps. */
 	double ts_peer_;        /* the most recent timestamp the peer sent */
 	double ts_echo_;        /* the most recent timestamp the peer echoed */
@@ -227,35 +302,7 @@ protected:
 	int ts_option_;		/* use RFC1323-like timestamps? */
 	/* End of timestamps. */
 
-	/* connection and packet dynamics */
-	virtual void output(int seqno, int reason = 0);
-	virtual void send_much(int force, int reason, int maxburst = 0);
-	virtual void newtimer(Packet*);
-	virtual void dupack_action();		/* do this on dupacks */
-	virtual void send_one();		/* do this on 1-2 dupacks */
-	double linear(double x, double x_1, double y_1, double x_2, double y_2);
-	/* the "linear" function is for experimental highspeed TCP */
-	void opencwnd();
-
-	void slowdown(int how);			/* reduce cwnd/ssthresh */
-	void ecn(int seqno);		/* react to quench */
-	virtual void set_initial_window();	/* set IW */
-	double initial_window();		/* what is IW? */
-	void reset();
-	void newack(Packet*);
-	void tcp_eln(Packet *pkt); /* reaction to ELN (usually wireless) */
-	void finish(); /* called when the connection is terminated */
-	void reset_qoption();	/* for QOption with EnblRTTCtr_ */
-	void rtt_counting();	/* for QOption with EnblRTTCtr_ */
-	int network_limited();	/* Sending limited by network? */
-	double limited_slow_start(double cwnd, double max_ssthresh, double increment);
-				/* Limited slow-start for high windows */
-	virtual int numdupacks(double cwnd); 	/* for getting numdupacks_ */
-	virtual void processQuickStart(Packet *pkt);
-	virtual void endQuickStart();
-	int lossQuickStart();
-
-	/* Helper functions. Currently used by tcp-asym */
+	/* Helper functions.  Used by tcp-asym */
 	virtual void output_helper(Packet*) { return; }
 	virtual void send_helper(int) { return; }
 	virtual void send_idle_helper() { return; }
@@ -287,37 +334,18 @@ protected:
 	int rfc2988_;		/* Use updated RFC 2988 timers */
 	/* End of timers. */ 
 
-	double boot_time_;	/* where between 'ticks' this sytem came up */
-	double overhead_;
-	double wnd_;
-	double wnd_const_;
-	double wnd_th_;		/* window "threshold" */
-	double wnd_init_;
-	double wnd_restart_;
-	double tcp_tick_;	/* clock granularity */
-	int wnd_option_;
-	int wnd_init_option_;   /* 1 for using wnd_init_ */
-				/* 2 for using large initial windows */
-	double decrease_num_;   /* factor for multiplicative decrease */
-	double increase_num_;   /* factor for additive increase */
-	int tcpip_base_hdr_size_;  /* size of base TCP/IP header */
-	int ts_resetRTO_;	/* Un-backoff RTO after any valid RTT, */
-				/*   including from a retransmitted pkt?  */
-				/* The old version was "false". */
-				/* But "true" gives better performance, and */
-                                /* seems conformant with RFC 2988. */
-	int maxcwnd_;		/* max # cwnd can ever be */
-        int numdupacks_;	/* dup ACKs before fast retransmit */
-	int numdupacksFrac_;	/* for a larger numdupacks_ with large */
-				/* windows */
-	double maxrto_;		/* max value of an RTO */
-	double minrto_;         /* min value of an RTO */
 
 	/* For modeling SYN and SYN/ACK packets. */
 	int syn_;		/* 1 for modeling SYN/ACK exchange */
 	int delay_growth_;  	/* delay opening cwnd until 1st data recv'd */
+        int max_connects_;  	/* max number of transmits for syn packet */
+				/* -1 to allow infinite number of transmits */
 	/* End of modeling SYN and SYN/ACK packets. */
 
+	/* Dynamic state for SYN packet retransmissions. */
+	int syn_connects_;	/* number of transmits of syn packet */
+	/* End of dynamic state for SYN packet retransmissions. */
+
 	/* F-RTO */
 	int frto_enabled_;	/* != 0 to enable F-RTO */
 	int sfrto_enabled_;	/* != 0 to enabled SACK-based F-RTO */
@@ -341,6 +369,8 @@ protected:
 				 * old ECN implementation, which never
 				 * reduced the congestion window below
 				 * one packet. */ 
+	int bugfix_ss_;		// 1 to use window of one when SYN
+				//  packet is dropped
 	/* End of parameters for backwards compatility. */
 
 	/* Parameters for alternate congestion control mechanisms. */
@@ -352,29 +382,6 @@ protected:
 	/* End of parameters for alternate congestion control mechanisms. */
 
 	FILE *plotfile_;
-	/*
-	 * Dynamic state.
-	 */
-	TracedInt dupacks_;	/* number of duplicate acks */
-	TracedInt curseq_;	/* highest seqno "produced by app" */
-	TracedInt highest_ack_;	/* not frozen during Fast Recovery */
-	TracedDouble cwnd_;	/* current window */
-	TracedInt ssthresh_;	/* slow start threshold */
-	TracedInt maxseq_;	/* used for Karn algorithm */
-				/* highest seqno sent so far */
-	int last_ack_;		/* largest consecutive ACK, frozen during
-				 *		Fast Recovery */
-	int recover_;		/* highest pkt sent before dup acks, */
-				/*   timeout, or source quench/ecn */
-	int last_cwnd_action_;	/* CWND_ACTION_{TIMEOUT,DUPACK,ECN} */
-	int count_;		/* used in window increment algorithms */
-	int rtt_active_;	/* 1 if a rtt sample is pending */
-	int rtt_seq_;		/* seq # of timed seg if rtt_active_ is 1 */
-	double rtt_ts_;		/* time at which rtt_seq_ was sent */
-	double firstsent_;	/* When first packet was sent  --Allman */
-	double lastreset_;	/* W.N. Last time connection was reset - for */
-				/* detecting pkts from previous incarnations */
-	int closed_;            /* whether this connection has closed */
 
 	/* Dynamic state used for alternate congestion control mechanisms */
 	double awnd_;		/* averaged window */
@@ -430,15 +437,19 @@ protected:
 	int SetCWRonRetransmit_;  /* True to allow setting CWR on */
 				  /*  retransmitted packets.   Affects */
 				  /*  performance for Reno with ECN.  */
+	int use_rtt_;	     /* Use RTT for timeout for ECN-marked SYN-ACK */
 	/* end of ECN */
 
 	/* used for Explicit Loss Notification */
+	void tcp_eln(Packet *pkt); /* reaction to ELN (usually wireless) */
         int eln_;               /* Explicit Loss Notification (wireless) */
         int eln_rxmit_thresh_;  /* Threshold for ELN-triggered rxmissions */
         int eln_last_rxmit_;    /* Last packet rxmitted due to ELN info */
 	/* end of Explicit Loss Notification */
 
-	/* for experimental high-speed TCP */
+	/* for High-Speed TCP, RFC 3649 */
+	double linear(double x, double x_1, double y_1, double x_2, double y_2);
+	  /* the "linear" function is for experimental highspeed TCP */
 	/* These four parameters define the HighSpeed response function. */
 	int low_window_;	/* window for turning on high-speed TCP */
 	int high_window_;	/* target window for new response function */
@@ -454,8 +465,11 @@ protected:
 	hstcp hstcp_;		/* HighSpeed TCP variables */
         /* end of section for experimental high-speed TCP */
 
-	/* for Quick-Start, experimental. */
-	int rate_request_;      /* Rate request in Kbps, for QuickStart.  */
+	/* for Quick-Start, RFC 4782 */
+	virtual void processQuickStart(Packet *pkt);
+	virtual void endQuickStart();
+	int lossQuickStart();
+	int rate_request_;      /* Rate request in KBps, for QuickStart.  */
 	int qs_enabled_;        /* to enable QuickStart. */
 	int qs_requested_;
 	int qs_approved_;
@@ -467,7 +481,7 @@ protected:
 	int qs_request_mode_;   /* 1 = Try to avoid unnecessary QS requests
 				   for short flows. Use qs_rtt_ as the RTT
 				   used in window calculation.
-				   2 = Always request 'rate_request_' bytes,
+				   Other: Always request 'rate_request_' bytes,
 				   regardless of flow size */
 	int qs_thresh_;         /* Do not use QS if there are less data to send
 				   than this. Applies only if
@@ -476,6 +490,7 @@ protected:
 				   in order to be able to determine how much
 				   it needs for rate request with given amount
 				   of data to send. milliseconds. */
+	int print_request_;	/* true to print Quick-Start request */
 	int ttl_diff_;
         /* end of section for Quick-Start. */
 
@@ -495,6 +510,8 @@ protected:
 	/* TCP quiescence, reducing cwnd after an idle period */
 	void process_qoption_after_send() ;
 	void process_qoption_after_ack(int seqno) ;
+	void reset_qoption();	/* for QOption with EnblRTTCtr_ */
+	void rtt_counting();	/* for QOption with EnblRTTCtr_ */
 	int QOption_ ; /* TCP quiescence option */
 	int EnblRTTCtr_ ; /* are we using a corase grained timer? */
 	int T_full ; /* last time the window was full */
