@@ -34,7 +34,7 @@
  * Ported from CMU/Monarch's code, appropriate copyright applies.
  * nov'98 -Padma.
  *
- * $Header: /nfs/jade/vint/CVSROOT/ns-2/trace/cmu-trace.cc,v 1.88 2005/10/08 19:16:16 tomh Exp $
+ * $Header: /cvsroot/nsnam/ns-2/trace/cmu-trace.cc,v 1.94 2009/01/15 06:23:49 tom_henderson Exp $
  */
 
 #include <packet.h>
@@ -51,6 +51,8 @@
 #include <tora/tora_packet.h> //TORA
 #include <imep/imep_spec.h>         // IMEP
 #include <aodv/aodv_packet.h> //AODV
+// AOMDV patch
+#include <aomdv/aomdv_packet.h>
 #include <cmu-trace.h>
 #include <mobilenode.h>
 #include <simulator.h>
@@ -63,6 +65,39 @@
 #include "diffusion/diff_header.h" // DIFFUSION -- Chalermek
 
 
+PacketTracer::PacketTracer() : next_(0)
+{
+}
+void PacketTracer::setNext(PacketTracer *next)
+{
+	next_ = next;
+}
+
+PacketTracer::~PacketTracer()
+{
+}
+
+PacketTracer *PacketTracer::getNext()
+{
+	return next_;
+}
+
+int PacketTracer::format_unknow(Packet *p, int offset, BaseTrace *pt, int newtrace)
+{
+	return (format(p, offset, pt, newtrace) || (next_ && next_->format_unknow(p, offset, pt, newtrace)));
+}
+
+PacketTracer *CMUTrace::pktTrc_ = 0;
+
+void CMUTrace::addPacketTracer(PacketTracer *pt)
+{
+	if(!pt)
+		return;
+	pt->setNext(pktTrc_);
+	pktTrc_ = pt;
+}
+
+
 //#define LOG_POSITION
 
 //extern char* pt_names[];
@@ -93,6 +128,9 @@ CMUTrace::CMUTrace(const char *s, char t) : Trace(t)
 	else if(strcmp(tracename, "TRP") == 0) {
                 tracetype = TR_ROUTER;
         }
+        else if(strcmp(tracename, "PHY") == 0) {
+                tracetype = TR_PHY;
+        }
         else if(strcmp(tracename, "MAC") == 0) {
                 tracetype = TR_MAC;
         }
@@ -230,7 +268,6 @@ CMUTrace::format_mac_common(Packet *p, const char *why, int offset)
         x = 0.0, y = 0.0, z = 0.0;
         node_->getLoc(&x, &y, &z);
 #endif
-
 	sprintf(pt_->buffer() + offset,
 #ifdef LOG_POSITION
 		"%c %.9f %d (%6.2f %6.2f) %3s %4s %d %s %d ",
@@ -250,9 +287,10 @@ CMUTrace::format_mac_common(Packet *p, const char *why, int offset)
                 ch->uid(),                      // identifier for this event
 		
 		((ch->ptype() == PT_MAC) ? (
+		  (mh->dh_fc.fc_type == MAC_Type_Control) ? (
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_RTS) ? "RTS"  :
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_CTS) ? "CTS"  :
-		  (mh->dh_fc.fc_subtype == MAC_Subtype_ACK) ? "ACK"  :
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_ACK) ? "ACK":
 		  //<zheng: add for 802.15.4>
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_Beacon) ? "BCN"  :		//Beacon
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_AssoReq) ? "CM1"  :	//CMD: Association request
@@ -264,7 +302,15 @@ CMUTrace::format_mac_common(Packet *p, const char *why, int offset)
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_BconReq) ? "CM7"  :	//CMD: Beacon request
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_CoorRea) ? "CM8"  :	//CMD: Coordinator realignment
 		  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_GTSReq) ? "CM9"  :	//CMD: GTS request
-		  //</zheng: add for 802.15.4>
+	          "UNKN") :
+		   (mh->dh_fc.fc_type == MAC_Type_Management) ? (
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_80211_Beacon) ? "BCN"  :
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_AssocReq) ? "ACRQ"  :
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_AssocRep) ? "ACRP"  : 
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_Auth) ? "AUTH"  :
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_ProbeReq) ? "PRRQ"  :
+		  (mh->dh_fc.fc_subtype == MAC_Subtype_ProbeRep) ? "PRRP"  :
+		  "UNKN") :
 		  "UNKN") :
 		 (ch->ptype() == PT_SMAC) ? (
 		  (sh->type == RTS_PKT) ? "RTS" :
@@ -277,6 +323,12 @@ CMUTrace::format_mac_common(Packet *p, const char *why, int offset)
 	
 	offset = strlen(pt_->buffer());
 
+	if(tracetype == TR_PHY) {
+		format_phy(p, offset);
+		offset = strlen(pt_->buffer());
+		return;
+	}
+
 	if (strncmp (mactype, "Mac/SMAC", 8) == 0) {
 		format_smac(p, offset);
 	} else {
@@ -287,17 +339,40 @@ CMUTrace::format_mac_common(Packet *p, const char *why, int offset)
 
 	if (thisnode) {
 		if (thisnode->energy_model()) {
+			// log detailed energy consumption
+			// total energy and breakdown in idle, sleep, transmit and receive modes
 			sprintf(pt_->buffer() + offset,
-				"[energy %f] ",
-				thisnode->energy_model()->energy());
+				"[energy %f ei %.3f es %.3f et %.3f er %.3f] ",
+				thisnode->energy_model()->energy(),
+				thisnode->energy_model()->ei(),
+				thisnode->energy_model()->es(),				
+				thisnode->energy_model()->et(),
+				thisnode->energy_model()->er());				
 		}
         }
 }
 
+void
+CMUTrace::format_phy(Packet *p, int offset)
+{
+	sprintf(pt_->buffer() + offset, " ");
+}
+
+
 void
 CMUTrace::format_mac(Packet *p, int offset)
 {
 	struct hdr_mac802_11 *mh = HDR_MAC802_11(p);
+	struct hdr_cmn *ch = HDR_CMN(p);
+	// This function assumes in some places that mh->dh_body points
+	// to an ethertype, which may not be true and causes some portability
+	// problems, so we zero the printing of this field in some cases
+	bool print_ether_type = true;
+	if ( (ch->ptype() == PT_MAC) && 
+	     ( (mh->dh_fc.fc_type == MAC_Type_Control) ||
+	       (mh->dh_fc.fc_type == MAC_Type_Management))) {
+		print_ether_type = false;
+	} 
 	
 	if (pt_->tagged()) {
 		sprintf(pt_->buffer() + offset,
@@ -311,7 +386,7 @@ CMUTrace::format_mac(Packet *p, int offset)
                        ETHER_ADDR(mh->dh_ta),          // MAC: destination
 
 
-			GET_ETHER_TYPE(mh->dh_body));	// MAC: type
+			print_ether_type ? GET_ETHER_TYPE(mh->dh_body) : 0);	// MAC: type
 	} else if (newtrace_) {
 		sprintf(pt_->buffer() + offset, 
 			"-Ma %x -Md %x -Ms %x -Mt %x ",
@@ -324,7 +399,7 @@ CMUTrace::format_mac(Packet *p, int offset)
 	   		ETHER_ADDR(mh->dh_ra),
 	                   ETHER_ADDR(mh->dh_ta),
 
-			GET_ETHER_TYPE(mh->dh_body));
+			print_ether_type ? GET_ETHER_TYPE(mh->dh_body) : 0);
 	} else {
 		sprintf(pt_->buffer() + offset,
 			" [%x %x %x %x] ",
@@ -336,9 +411,7 @@ CMUTrace::format_mac(Packet *p, int offset)
 			//ETHER_ADDR(mh->dh_sa),
 			ETHER_ADDR(mh->dh_ra),
                         ETHER_ADDR(mh->dh_ta),
-
-
-			GET_ETHER_TYPE(mh->dh_body));
+			print_ether_type ? GET_ETHER_TYPE(mh->dh_body) : 0);
 	}
 }
 
@@ -880,6 +953,114 @@ CMUTrace::format_aodv(Packet *p, int offset)
         }
 }
 
+// AOMDV patch
+void
+CMUTrace::format_aomdv(Packet *p, int offset)
+{
+	struct hdr_aomdv *ah = HDR_AOMDV(p);
+	struct hdr_aomdv_request *rq = HDR_AOMDV_REQUEST(p);
+	struct hdr_aomdv_reply *rp = HDR_AOMDV_REPLY(p);
+	
+	
+	switch(ah->ah_type) {
+		case AOMDVTYPE_RREQ:
+			
+			if (pt_->tagged()) {
+				sprintf(pt_->buffer() + offset,
+						  "-aomdv:t %x -aomdv:h %d -aomdv:b %d -aomdv:d %d "
+						  "-aomdv:ds %d -aomdv:s %d -aomdv:ss %d "
+						  "-aomdv:c REQUEST ",
+						  rq->rq_type,
+						  rq->rq_hop_count,
+						  rq->rq_bcast_id,
+						  rq->rq_dst,
+						  rq->rq_dst_seqno,
+						  rq->rq_src,
+						  rq->rq_src_seqno);
+			} else if (newtrace_) {
+				
+				sprintf(pt_->buffer() + offset,
+						  "-P aomdv -Pt 0x%x -Ph %d -Pb %d -Pd %d -Pds %d -Ps %d -Pss %d -Pc REQUEST ",
+						  rq->rq_type,
+						  rq->rq_hop_count,
+						  rq->rq_bcast_id,
+						  rq->rq_dst,
+						  rq->rq_dst_seqno,
+						  rq->rq_src,
+						  rq->rq_src_seqno);
+				
+				
+			} else {
+				
+				sprintf(pt_->buffer() + offset,
+						  "[0x%x %d %d [%d %d] [%d %d]] (REQUEST)",
+						  rq->rq_type,
+						  rq->rq_hop_count,
+						  rq->rq_bcast_id,
+						  rq->rq_dst,
+						  rq->rq_dst_seqno,
+						  rq->rq_src,
+						  rq->rq_src_seqno);
+			}
+			break;
+			
+		case AOMDVTYPE_RREP:
+		case AOMDVTYPE_HELLO:
+		case AOMDVTYPE_RERR:
+			
+			if (pt_->tagged()) {
+				sprintf(pt_->buffer() + offset,
+						  "-aomdv:t %x -aomdv:h %d -aomdv:d %d -admov:ds %d "
+						  "-aomdv:l %f -aomdv:c %s ",
+						  rp->rp_type,
+						  rp->rp_hop_count,
+						  rp->rp_dst,
+						  rp->rp_dst_seqno,
+						  rp->rp_lifetime,
+						  rp->rp_type == AOMDVTYPE_RREP ? "REPLY" :
+						  (rp->rp_type == AOMDVTYPE_RERR ? "ERROR" :
+							"HELLO"));
+			} else if (newtrace_) {
+				
+				sprintf(pt_->buffer() + offset,
+						  "-P aomdv -Pt 0x%x -Ph %d -Pd %d -Pds %d -Pl %f -Pc %s ",
+						  rp->rp_type,
+						  rp->rp_hop_count,
+						  rp->rp_dst,
+						  rp->rp_dst_seqno,
+						  rp->rp_lifetime,
+						  rp->rp_type == AOMDVTYPE_RREP ? "REPLY" :
+						  (rp->rp_type == AOMDVTYPE_RERR ? "ERROR" :
+							"HELLO"));
+	        } else {
+				  sprintf(pt_->buffer() + offset,
+							 "[0x%x %d [%d %d] %f] (%s) [%d %d]",
+							 rp->rp_type,
+							 rp->rp_hop_count,
+							 rp->rp_dst,
+							 rp->rp_dst_seqno,
+							 rp->rp_lifetime,
+							 rp->rp_type == AODVTYPE_RREP ? "REPLY" :
+							 (rp->rp_type == AODVTYPE_RERR ? "ERROR" :
+							  "HELLO"),
+							 rp->rp_bcast_id,
+							 rp->rp_first_hop
+							 );
+			  }
+			break;
+			
+		default:
+#ifdef WIN32
+			fprintf(stderr,
+					  "CMUTrace::format_aomdv: invalid AODV packet type\n");
+#else
+			fprintf(stderr,
+					  "%s: invalid AOMDV packet type\n", __FUNCTION__);
+#endif
+			abort();
+	}
+}
+
 void
 CMUTrace::nam_format(Packet *p, int offset)
 {
@@ -912,6 +1093,8 @@ CMUTrace::nam_format(Packet *p, int offset)
 	  (mh->dh_fc.fc_subtype == MAC_Subtype_CTS) ? "CTS"  :
 	  (mh->dh_fc.fc_subtype == MAC_Subtype_ACK) ? "ACK"  :
 	  (mh->dh_fc.fc_subtype == MAC_Subtype_Beacon) ? "BCN"  :		//Beacon
+	  (mh->dh_fc.fc_subtype == MAC_Subtype_AssocReq) ? "ACRQ"  :
+          (mh->dh_fc.fc_subtype == MAC_Subtype_AssocRep) ? "ACRP"  :
 	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_AssoReq) ? "CM1"  :	//CMD: Association request
 	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_AssoRsp) ? "CM2"  :	//CMD: Association response
 	  (mh->dh_fc.fc_subtype == MAC_Subtype_Command_DAssNtf) ? "CM3"  :	//CMD: Disassociation notification
@@ -1144,6 +1327,10 @@ void CMUTrace::format(Packet* p, const char *why)
 		case PT_AODV:
 			format_aodv(p, offset);
 			break;
+		// AOMDV patch
+		case PT_AOMDV:
+			format_aomdv(p, offset);
+			break;
 		case PT_TORA:
                         format_tora(p, offset);
                         break;
@@ -1175,6 +1362,10 @@ void CMUTrace::format(Packet* p, const char *why)
 		case PT_PING:
 			break;
 		default:
+
+			if(pktTrc_ && pktTrc_->format_unknow(p, offset, pt_, newtrace_))
+				break;
+
 		/*<zheng: del -- there are many more new packet types added, like PT_EXP (poisson traffic belongs to this type)>
 			fprintf(stderr, "%s - invalid packet type (%s).\n",
 				__PRETTY_FUNCTION__, packet_info.name(ch->ptype()));
